<?php

namespace Drupal\commerce_omnikassa\Plugin\Commerce\PaymentGateway;

use Drupal\commerce_order\Entity\OrderInterface;
use Drupal\commerce_payment\Plugin\Commerce\PaymentGateway\OffsitePaymentGatewayBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Messenger\MessengerInterface;
use Symfony\Component\HttpFoundation\Request;
use nl\rabobank\gict\payments_savings\omnikassa_sdk\model\signing\SigningKey;
use nl\rabobank\gict\payments_savings\omnikassa_sdk\model\response\PaymentCompletedResponse;

/**
 * Provides the Rabo Omnikassa payment gateway.
 *
 * @CommercePaymentGateway(
 *   id = "omnikassa_redirect_checkout",
 *   label = @Translation("Omnikassa (Redirect to the Rabo Omnikassa)"),
 *   display_label = @Translation("Rabo Omnikassa"),
 *    forms = {
 *     "offsite-payment" = "Drupal\commerce_omnikassa\PluginForm\OffsiteRedirect\RedirectCheckoutForm",
 *   },
 * )
 */
class RaboOmnikassa extends OffsitePaymentGatewayBase {

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return [
      'omnikassa_token' => '',
      'omnikassa_signing_key' => '',
    ] + parent::defaultConfiguration();
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $form = parent::buildConfigurationForm($form, $form_state);

    $form['omnikassa_token'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Token'),
      '#description' => $this->t('Enter the token from your Dashboard.'),
      '#default_value' => $this->configuration['omnikassa_token'],
      '#required' => TRUE,
    ];

    $form['omnikassa_signing_key'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Signing key'),
      '#description' => $this->t('The signing key from your Dashboard.'),
      '#default_value' => $this->configuration['omnikassa_signing_key'],
      '#required' => TRUE,
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state) {
    parent::submitConfigurationForm($form, $form_state);
    $values = $form_state->getValue($form['#parents']);
    $this->configuration['omnikassa_token'] = $values['omnikassa_token'];
    $this->configuration['omnikassa_signing_key'] = $values['omnikassa_signing_key'];
  }

  /**
   * {@inheritdoc}
   */
  public function onReturn(OrderInterface $order, Request $request) {
    // Validate response.
    $orderId = $request->get('order_id');
    $status = $request->get('status');
    $signature = $request->get('signature');
    $signingKey = new SigningKey(base64_decode($this->configuration['omnikassa_signing_key']));
    // Validate and sanitize response with PaymentCompletedResponse.
    $paymentCompletedResponse = PaymentCompletedResponse::createInstance($orderId, $status, $signature, $signingKey);
    if (!$paymentCompletedResponse) {
      \Drupal::logger('commerce_omnikassa')->error('Payment response validation failed.
      Orderid: @order_id.
      Status: @status.
      Signature: @signature',
        [
          '@order_id' => $orderId,
          '@status' => $status,
          '@signature' => $signature,
        ]
      );
    }
    else {
      // Check if order was completed.
      if ($paymentCompletedResponse->getStatus() != 'COMPLETED') {
        $this->messenger()->addMessage($this->t('You have canceled checkout at Rabo Omnikassa.
        Please contact us how to complete your order.'),
          MessengerInterface::TYPE_WARNING);
      }
      else {
        $this->messenger()->addMessage($this->t('Your payment was successful!'));
      }
      // Record the payment anyway. Status will reflect the status.
      $payment_storage = $this->entityTypeManager->getStorage('commerce_payment');
      $payment = $payment_storage->create([
        'state' => $paymentCompletedResponse->getStatus(),
        'amount' => $order->getBalance(),
        'payment_gateway' => $this->parentEntity->id(),
        'order_id' => $order->id(),
        'remote_id' => $paymentCompletedResponse->getOrderID(),
        'remote_state' => $paymentCompletedResponse->getStatus(),
      ]);
      $payment->save();
    }

  }

}
