<?php

namespace Drupal\commerce_omnikassa\PluginForm\OffsiteRedirect;

use Drupal\commerce_payment\PluginForm\PaymentOffsiteForm as BasePaymentOffsiteForm;
use Drupal\commerce_payment\Entity\PaymentInterface;
use Drupal\Core\Form\FormStateInterface;
use nl\rabobank\gict\payments_savings\omnikassa_sdk\model\Money;
use nl\rabobank\gict\payments_savings\omnikassa_sdk\model\OrderItem;
use nl\rabobank\gict\payments_savings\omnikassa_sdk\model\PaymentBrand;
use nl\rabobank\gict\payments_savings\omnikassa_sdk\model\PaymentBrandForce;
use nl\rabobank\gict\payments_savings\omnikassa_sdk\model\ProductType;
use nl\rabobank\gict\payments_savings\omnikassa_sdk\model\request\MerchantOrder;
use nl\rabobank\gict\payments_savings\omnikassa_sdk\model\Address;
use nl\rabobank\gict\payments_savings\omnikassa_sdk\model\CustomerInformation;
use nl\rabobank\gict\payments_savings\omnikassa_sdk\model\VatCategory;
use nl\rabobank\gict\payments_savings\omnikassa_sdk\model\Environment;
use nl\rabobank\gict\payments_savings\omnikassa_sdk\model\signing\SigningKey;
use nl\rabobank\gict\payments_savings\omnikassa_sdk\endpoint\Endpoint;

/**
 * {@inheritDoc}
 */
class RedirectCheckoutForm extends BasePaymentOffsiteForm {

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $form = parent::buildConfigurationForm($form, $form_state);

    /** @var \Drupal\commerce_payment\Entity\PaymentInterface $payment */
    $payment = $this->entity;

    /** @var \Drupal\commerce_payment\Plugin\Commerce\PaymentGateway\OffsitePaymentGatewayInterface $payment_gateway_plugin */
    $payment_gateway_plugin = $payment->getPaymentGateway()->getPlugin();
    $configuration = $payment_gateway_plugin->getConfiguration();

    $order = $this->buildOmnikassaOrder($form, $payment);

    if ($configuration['mode'] == 'test') {
      $omnikassa_environment = Environment::SANDBOX;
    }
    else {
      $omnikassa_environment = Environment::PRODUCTION;
    }
    $signingKey = new SigningKey(base64_decode($configuration['omnikassa_signing_key']));
    $inMemoryTokenProvider = new InMemoryTokenProvider($configuration['omnikassa_token']);
    $endpoint = Endpoint::createInstance($omnikassa_environment, $signingKey, $inMemoryTokenProvider);
    $redirectUrl = $endpoint->announceMerchantOrder($order);

    $data = [];

    return $this->buildRedirectForm(
      $form,
      $form_state,
      $redirectUrl,
      $data,
      self::REDIRECT_GET
    );

  }

  /**
   * Function creates a valid Rabo Omnikassa order.
   *
   * @param array $form
   *   The form.
   * @param \Drupal\commerce_payment\Entity\PaymentInterface $payment
   *   Payment provider configuration.
   *
   * @return \nl\rabobank\gict\payments_savings\omnikassa_sdk\model\request\MerchantOrder
   *   Valid order.
   *
   * @throws \Drupal\Core\TypedData\Exception\MissingDataException
   */
  private function buildOmnikassaOrder(array $form,
                                       PaymentInterface $payment) {

    /** @var \Drupal\commerce_payment\Plugin\Commerce\PaymentGateway\OffsitePaymentGatewayInterface $payment_gateway_plugin */
    $payment_gateway_plugin = $payment->getPaymentGateway()->getPlugin();
    $configuration = $payment_gateway_plugin->getConfiguration();
    $order = $payment->getOrder();

    foreach ($order->getItems() as $key => $order_item) {

      $product_variation = $order_item->getPurchasedEntity();

      $order_items[] = OrderItem::createFrom([
        'id' => $product_variation->id(),
        'name' => $product_variation->getTitle(),
        'description' => $product_variation->getTitle(),
        'quantity' => $order_item->getQuantity(),
        'amount' => Money::fromDecimal('EUR', $product_variation->getPrice()->getNumber()),
        'category' => ProductType::DIGITAL,
      ]);

      /* TODO:
      - category can be DIGITAL or PHYSICAL
      - Should we add VAT?
      'tax' => Money::fromDecimal('EUR', 00.00),
      'vatCategory'
      1 = Hoog (momenteel 21%),
      2 = Laag (vanaf 1 januari 2019 is dit 9%),
      3 = Nul (0%),
      4 = Geen (vrijgesteld van BTW)
       */

    }

    // Get addresses (Shipping and Billing).
    $profiles = $order->collectProfiles();
    if (isset($profiles['billing'])) {
      $billing_profile = $profiles['billing'];
      $billing_address = $billing_profile->get('address')->first();
      $billing_details = Address::createFrom([
        'firstName' => $billing_address->getGivenName(),
        'middleName' => '',
        'lastName' => $billing_address->getFamilyName(),
        'street' => $billing_address->getAddressLine1() . ' ' . $billing_address->getAddressLine2(),
        'postalCode' => $billing_address->getPostalCode(),
        'city' => $billing_address->getLocality(),
        'countryCode' => $billing_address->getCountryCode(),
        'houseNumber' => '',
        'houseNumberAddition' => '',
      ]);
    }
    if (isset($profiles['shipping'])) {
      $shipping_profile = $profiles['shipping'];
      $shipping_address = $shipping_profile->get('address')->first();
      $shipping_details = Address::createFrom([
        'firstName' => $shipping_address->getGivenName(),
        'middleName' => '',
        'lastName' => $shipping_address->getFamilyName(),
        'street' => $shipping_address->getAddressLine1() . ' ' . $shipping_address->getAddressLine2(),
        'postalCode' => $shipping_address->getPostalCode(),
        'city' => $shipping_address->getLocality(),
        'countryCode' => $shipping_address->getCountryCode(),
        'houseNumber' => '',
        'houseNumberAddition' => '',
      ]);
    }
    else {
      // If billing address isset, and shipping is not, they are the same.
      if (isset($billing_details)) {
        $shipping_details = $billing_details;
      }
    }

    $customerInformation = CustomerInformation::createFrom([
      'emailAddress' => $order->getEmail(),
    ]);

    $data = [
      'merchantOrderId' => $payment->getOrderId(),
      'description' => 'Troost.nl order ' . $payment->getOrderId(),
      'amount' => Money::fromDecimal('EUR', $payment->getAmount()->getNumber()),
      'customerInformation' => $customerInformation,
      'language' => 'NL',
      'merchantReturnURL' => $form['#return_url'],
      'paymentBrand' => PaymentBrand::IDEAL,
      'paymentBrandForce' => PaymentBrandForce::FORCE_ONCE,
    ];
    if (isset($shipping_details)) {
      $data['shippingDetail'] = $shipping_details;
    }
    if (isset($billing_details)) {
      $data['billingDetail'] = $billing_details;
    }
    if (isset($order_items)) {
      $data['orderItems'] = $order_items;
    }

    return MerchantOrder::createFrom($data);
  }

}
